 MODULE dvrp_color

    USE dvrp_variables

    IMPLICIT NONE

 CONTAINS

    SUBROUTINE color_dvrp( value, color )

       REAL, INTENT(IN)  ::  value
       REAL, INTENT(OUT) ::  color(4)

       REAL              ::  scale

       scale = ( value - slicer_range_limits_dvrp(1,islice_dvrp) ) / &
               ( slicer_range_limits_dvrp(2,islice_dvrp) -           &
                 slicer_range_limits_dvrp(1,islice_dvrp) )

       scale = MODULO( 180.0 + 180.0 * scale, 360.0 )

       color = (/ scale, 0.5, 1.0, 0.0 /)

    END SUBROUTINE color_dvrp

 END MODULE dvrp_color


 RECURSIVE SUBROUTINE data_output_dvrp

!------------------------------------------------------------------------------!
! Actual revisions:
! -----------------
! TEST: write statements
!
! Former revisions:
! -----------------
! $Id: data_output_dvrp.f90 13 2007-02-14 12:15:07Z raasch $
! RCS Log replace by Id keyword, revision history cleaned up
!
! Revision 1.13  2006/02/23 10:25:12  raasch
! Former routine plot_dvrp renamed data_output_dvrp,
! Only a fraction of the particles may have a tail,
! pl.. replaced by do.., %size renamed %dvrp_psize
!
! Revision 1.1  2000/04/27 06:27:17  raasch
! Initial revision
!
!
! Description:
! ------------
! Plot of isosurface, particles and slicers with dvrp-software
!------------------------------------------------------------------------------!
#if defined( __dvrp_graphics )

    USE arrays_3d
    USE cloud_parameters
    USE cpulog
    USE DVRP
    USE dvrp_color
    USE dvrp_variables
    USE grid_variables
    USE indices
    USE interfaces
    USE particle_attributes
    USE pegrid
    USE control_parameters

    IMPLICIT NONE

    CHARACTER (LEN=2) ::  section_chr
    CHARACTER (LEN=6) ::  output_variable
    INTEGER ::  i, j, k, l, m, n, nn, section_mode, tv, vn
    INTEGER, DIMENSION(:), ALLOCATABLE ::  p_c, p_t
    REAL    ::  center(3), distance, slicer_position, surface_value
    REAL, DIMENSION(:),     ALLOCATABLE ::  psize, p_x, p_y, p_z
    REAL, DIMENSION(:,:,:), ALLOCATABLE ::  local_pf


    WRITE ( 9, * ) '*** myid=', myid, ' Anfang data_output_dvrp'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
    CALL cpu_log( log_point(27), 'data_output_dvrp', 'start' )

!
!-- Loop over all output modes choosed
    m           = 1
    tv          = 0  ! threshold counter
    islice_dvrp = 0  ! slice plane counter
    DO WHILE ( mode_dvrp(m) /= ' ' )
!
!--    Update of the steering variables
       IF ( .NOT. lock_steering_update )  THEN
!
!--       Set lock to avoid recursive calls of DVRP_STEERING_UPDATE
          lock_steering_update = .TRUE.
    WRITE ( 9, * ) '*** myid=', myid, ' data_output_dvrp: vor steering_update'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
          CALL DVRP_STEERING_UPDATE( m-1, data_output_dvrp )
    WRITE ( 9, * ) '*** myid=', myid, ' data_output_dvrp: nach steering_update'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
          lock_steering_update = .FALSE.
       ENDIF

!
!--    Determine the variable which shall be plotted (in case of slicers or
!--    isosurfaces)
       IF ( mode_dvrp(m)(1:10) == 'isosurface' )  THEN
          READ ( mode_dvrp(m), '(10X,I1)' )  vn
          output_variable = do3d(0,vn)
          tv = tv + 1
       ELSEIF ( mode_dvrp(m)(1:6) == 'slicer' )  THEN
          READ ( mode_dvrp(m), '(6X,I1)' )  vn
          output_variable = do2d(0,vn)
          l = MAX( 2, LEN_TRIM( do2d(0,vn) ) )
          section_chr = do2d(0,vn)(l-1:l)
          SELECT CASE ( section_chr )
             CASE ( 'xy' )
                section_mode = 2
                slicer_position = zu(MIN( slicer_position_dvrp(m), nz_do3d ))
             CASE ( 'xz' )
                section_mode = 1
                slicer_position = slicer_position_dvrp(m) * dy
             CASE ( 'yz' )
                section_mode = 0
                slicer_position = slicer_position_dvrp(m) * dx
          END SELECT
       ENDIF

!
!--    Select the plot mode (in case of isosurface or slicer only if user has
!--    defined a variable which shall be plotted; otherwise do nothing)
       IF ( mode_dvrp(m)(1:9) == 'particles' )  THEN

#if defined( __particles )
    WRITE ( 9, * ) '*** myid=', myid, ' data_output_dvrp: anfang particles'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
!
!--       DVRP-Calls for plotting particles:
          CALL cpu_log( log_point_s(28), 'dvrp_particles', 'start' )

!
!--       Definition of characteristics of particle material
          CALL DVRP_MATERIAL_RGB( m-1, 1, 0.1, 0.7, 0.1, 0.0 )

!
!--       Move particle coordinates to one-dimensional arrays
          IF ( .NOT. use_particle_tails )  THEN
!
!--          All particles are output
             ALLOCATE( psize(number_of_particles), p_t(number_of_particles), &
                       p_c(number_of_particles), p_x(number_of_particles),   &
                       p_y(number_of_particles), p_z(number_of_particles) )
             psize = 0.0;  p_t = 0;  p_c = 0.0;  p_x = 0.0;  p_y = 0.0
             p_z   = 0.0;
             psize = particles(1:number_of_particles)%dvrp_psize
             p_x   = particles(1:number_of_particles)%x * superelevation_x
             p_y   = particles(1:number_of_particles)%y * superelevation_y
             p_z   = particles(1:number_of_particles)%z * superelevation
             p_c   = particles(1:number_of_particles)%color
          ELSE
!
!--          Particles have a tail
             WRITE (9,*) '--- before ALLOCATE  simtime=',simulated_time,' #of_tails=', number_of_tails, &
                           ' max#of_tp=', maximum_number_of_tailpoints
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
             ALLOCATE( psize(number_of_tails), p_t(number_of_tails),      &
                       p_c(number_of_tails*maximum_number_of_tailpoints), &
                       p_x(number_of_tails*maximum_number_of_tailpoints), &
                       p_y(number_of_tails*maximum_number_of_tailpoints), &
                       p_z(number_of_tails*maximum_number_of_tailpoints) )
             WRITE (9,*) '--- after ALLOCATE'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
             psize = 0.0;  p_t = 0;  p_c = 0.0;  p_x = 0.0;  p_y = 0.0
             p_z   = 0.0;
             i = 0
             k = 0
             DO  n = 1, number_of_particles
                nn = particles(n)%tail_id
                IF ( nn /= 0 )  THEN
                   k = k + 1
                   IF ( simulated_time > 1338.0 )  THEN
                      WRITE (9,*) '--- particle ',n,' tail_id=',nn,' #of_tp=',particles(n)%tailpoints
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
                   ENDIF
                   DO  j = 1, particles(n)%tailpoints
                      i = i + 1
                      p_x(i) = particle_tail_coordinates(j,1,nn) * &
                                                                superelevation_x
                      p_y(i) = particle_tail_coordinates(j,2,nn) * &
                                                                superelevation_y
                      p_z(i) = particle_tail_coordinates(j,3,nn) * &
                                                                superelevation
                      p_c(i) = particle_tail_coordinates(j,4,nn)
                      IF ( simulated_time > 1338.0 )  THEN
                         WRITE (9,*) '--- tp= ',i,' x=',p_x(i),' y=',p_y(i), &
                                                 ' z=',p_z(i),' c=',p_c(i)
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
                      ENDIF
                   ENDDO
                   psize(k) = particles(n)%dvrp_psize
                   p_t(k)   = particles(n)%tailpoints - 1
                   IF ( simulated_time > 1338.0 )  THEN
                      WRITE (9,*) '--- t= ',k,' psize=',psize(k),' p_t=',p_t(k)
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
                   ENDIF
                ENDIF                
             ENDDO
             WRITE (9,*) '--- after locally storing the particle attributes'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
          ENDIF

!
!--       Compute and plot particles in dvr-format
          IF ( uniform_particles  .AND.  .NOT. use_particle_tails )  THEN
!
!--          All particles have the same color. Use simple routine to set
!--          the particle attributes (produces less output data)
             CALL DVRP_PARTICLES( m-1, p_x, p_y, p_z, psize )
          ELSE
!
!--          Set color definitions
             CALL user_dvrp_coltab( 'particles', 'none' )

             CALL DVRP_COLORTABLE_HLS( m-1, 0, interval_values_dvrp,     &
                                       interval_h_dvrp, interval_l_dvrp, &
                                       interval_s_dvrp, interval_a_dvrp )

             IF ( .NOT. use_particle_tails )  THEN
                CALL DVRP_PARTICLES( m-1, number_of_particles, p_x, p_y, p_z, &
                                     3, psize, p_c, p_t )
             ELSE
                WRITE (9,*) '--- before DVRP_PARTICLES'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
                CALL DVRP_PARTICLES( m-1, number_of_tails, p_x, p_y, p_z, 15, &
                                     psize, p_c, p_t )
                WRITE (9,*) '--- after DVRP_PARTICLES'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
             ENDIF
          ENDIF

          CALL DVRP_VISUALIZE( m-1, 3, dvrp_filecount )
    WRITE ( 9, * ) '*** myid=', myid, ' data_output_dvrp: ende particles'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif

          DEALLOCATE( psize, p_c, p_t, p_x, p_y, p_z )

          CALL cpu_log( log_point_s(28), 'dvrp_particles', 'stop' )
#endif


       ELSEIF ( ( mode_dvrp(m)(1:10) == 'isosurface'  .OR.   &
                  mode_dvrp(m)(1:6)  == 'slicer'           ) &
                  .AND.  output_variable /= ' ' )  THEN

!
!--       Create an intermediate array, properly dimensioned for plot-output 
          ALLOCATE( local_pf(nxl:nxr+1,nys:nyn+1,nzb:nz_do3d) )

!
!--       Move original array to intermediate array
          SELECT CASE ( output_variable )

             CASE ( 'u', 'u_xy', 'u_xz', 'u_yz' )
                DO  i = nxl, nxr+1
                   DO  j = nys, nyn+1
                      DO  k = nzb, nz_do3d
                         local_pf(i,j,k) = u(k,j,i)
                      ENDDO
                   ENDDO
                ENDDO
!
!--             Replace mirrored values at lower surface by real surface values
                IF ( output_variable == 'u_xz'  .OR. &
                     output_variable == 'u_yz' )  THEN
                   IF ( ibc_uv_b == 0 )  local_pf(:,:,nzb) = 0.0
                ENDIF


             CASE ( 'v', 'v_xy', 'v_xz', 'v_yz' )
                DO  i = nxl, nxr+1
                   DO  j = nys, nyn+1
                      DO  k = nzb, nz_do3d
                         local_pf(i,j,k) = v(k,j,i)
                      ENDDO
                   ENDDO
                ENDDO
!
!--             Replace mirrored values at lower surface by real surface values
                IF ( output_variable == 'v_xz'  .OR. &
                     output_variable == 'v_yz' )  THEN
                   IF ( ibc_uv_b == 0 )  local_pf(:,:,nzb) = 0.0
                ENDIF

             CASE ( 'w', 'w_xy', 'w_xz', 'w_yz' )
                DO  i = nxl, nxr+1
                   DO  j = nys, nyn+1
                      DO  k = nzb, nz_do3d
                         local_pf(i,j,k) = w(k,j,i)
                      ENDDO
                   ENDDO
                ENDDO

             CASE ( 'p', 'p_xy', 'p_xz', 'p_yz' )
                DO  i = nxl, nxr+1
                   DO  j = nys, nyn+1
                      DO  k = nzb, nz_do3d
                         local_pf(i,j,k) = p(k,j,i)
                      ENDDO
                   ENDDO
                ENDDO

             CASE ( 'pt', 'pt_xy', 'pt_xz', 'pt_yz' )
                IF ( .NOT. cloud_physics ) THEN
                   DO  i = nxl, nxr+1
                      DO  j = nys, nyn+1
                         DO  k = nzb, nz_do3d
                            local_pf(i,j,k) = pt(k,j,i)
                         ENDDO
                      ENDDO
                   ENDDO
                ELSE
                   DO  i = nxl, nxr+1
                      DO  j = nys, nyn+1
                         DO  k = nzb, nz_do3d
                            local_pf(i,j,k) = pt(k,j,i) + l_d_cp * pt_d_t(k) * &
                                                          ql(k,j,i)
                         ENDDO
                      ENDDO
                   ENDDO
                ENDIF

             CASE ( 'q', 'q_xy', 'q_xz', 'q_yz' )
                IF ( moisture  .OR.  passive_scalar )  THEN
                   DO  i = nxl, nxr+1
                      DO  j = nys, nyn+1
                         DO  k = nzb, nz_do3d
                            local_pf(i,j,k) = q(k,j,i)
                         ENDDO
                      ENDDO
                   ENDDO            
                ELSE
                   IF ( myid == 0 )  THEN
                      PRINT*, '+++ data_output_dvrp: if moisture/passive_scalar = ', & 
                              'FALSE output of ', output_variable,            &
                              'is not provided' 
                   ENDIF
                ENDIF
             
             CASE ( 'ql', 'ql_xy', 'ql_xz', 'ql_yz' )
                IF ( cloud_physics  .OR.  cloud_droplets )  THEN
                   DO  i = nxl, nxr+1
                      DO  j = nys, nyn+1
                         DO  k = nzb, nz_do3d
                            local_pf(i,j,k) = ql(k,j,i)
                         ENDDO
                      ENDDO
                   ENDDO
                ELSE
                   IF ( myid == 0 ) THEN
                      PRINT*, '+++ data_output_dvrp: if cloud_physics = FALSE ', & 
                              'output of ', output_variable, 'is not provided' 
                   ENDIF
                ENDIF

             CASE ( 'u*_xy' )
                DO  i = nxl, nxr+1
                   DO  j = nys, nyn+1
                      local_pf(i,j,nzb+1) = us(j,i)
                   ENDDO
                ENDDO
                slicer_position = zu(nzb+1)

             CASE ( 't*_xy' )
                DO  i = nxl, nxr+1
                   DO  j = nys, nyn+1
                      local_pf(i,j,nzb+1) = ts(j,i)
                   ENDDO
                ENDDO
                slicer_position = zu(nzb+1)


             CASE DEFAULT
                IF ( myid == 0 )  THEN
                   PRINT*,'+++ data_output_dvrp: no output possible for: ', &
                          output_variable
                ENDIF

          END SELECT


          IF ( mode_dvrp(m)(1:10) == 'isosurface' )  THEN

    WRITE ( 9, * ) '*** myid=', myid, ' data_output_dvrp: anfang isosurface'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
!
!--          DVRP-Calls for plotting isosurfaces:
             CALL cpu_log( log_point_s(26), 'dvrp_isosurface', 'start' )

!
!--          Definition of characteristics of isosurface material
!--          Preliminary settings for w!
             IF ( output_variable == 'w' )  THEN
                CALL DVRP_MATERIAL_RGB( m-1, 1, 0.3, 0.8, 0.3, 0.0 )
             ELSE
                CALL DVRP_MATERIAL_RGB( m-1, 1, 0.9, 0.9, 0.9, 0.0 )
             ENDIF

!
!--          Compute and plot isosurface in dvr-format
             CALL DVRP_DATA( m-1, local_pf, 1, nx_dvrp, ny_dvrp, nz_dvrp, &
                             cyclic_dvrp, cyclic_dvrp, cyclic_dvrp )
             CALL DVRP_THRESHOLD( m-1, threshold(tv) )
             CALL DVRP_VISUALIZE( m-1, 1, dvrp_filecount )
    WRITE ( 9, * ) '*** myid=', myid, ' data_output_dvrp: ende isosurface'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif

             CALL cpu_log( log_point_s(26), 'dvrp_isosurface', 'stop' )

          ELSEIF ( mode_dvrp(m)(1:6) == 'slicer' )  THEN

    WRITE ( 9, * ) '*** myid=', myid, ' data_output_dvrp: anfang slicer'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
!
!--          DVRP-Calls for plotting slicers:
             CALL cpu_log( log_point_s(27), 'dvrp_slicer', 'start' )

!
!--          Material and color definitions
             CALL DVRP_MATERIAL_RGB( m-1, 1, 0.0, 0.0, 0.0, 0.0 )

             islice_dvrp = islice_dvrp + 1
!             CALL DVRP_COLORFUNCTION( m-1, DVRP_CM_HLS, 25,                    &
!                                      slicer_range_limits_dvrp(:,islice_dvrp), &
!                                      color_dvrp )

             CALL user_dvrp_coltab( 'slicer', output_variable )

             CALL DVRP_COLORTABLE_HLS( m-1, 1, interval_values_dvrp,     &
                                       interval_h_dvrp, interval_l_dvrp, &
                                       interval_s_dvrp, interval_a_dvrp )

!
!--          Compute and plot slicer in dvr-format
             CALL DVRP_DATA( m-1, local_pf, 1, nx_dvrp, ny_dvrp, nz_dvrp, &
                             cyclic_dvrp, cyclic_dvrp, cyclic_dvrp )
             CALL DVRP_SLICER( m-1, section_mode, slicer_position )
             CALL DVRP_VISUALIZE( m-1, 2, dvrp_filecount )

             CALL cpu_log( log_point_s(27), 'dvrp_slicer', 'stop' )

    WRITE ( 9, * ) '*** myid=', myid, ' data_output_dvrp: ende slicer'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif
          ENDIF

          DEALLOCATE( local_pf )

       ENDIF

       m = m + 1

    ENDDO

    dvrp_filecount = dvrp_filecount + 1

    CALL cpu_log( log_point(27), 'data_output_dvrp', 'stop' )
    WRITE ( 9, * ) '*** myid=', myid, ' Ende data_output_dvrp'
#if defined( __ibm )
    CALL FLUSH_( 9 )
#elif defined( __lcmuk )  ||  defined( __lctit )  ||  defined( __nec )
    CALL FLUSH( 9 )
#endif

#endif
 END SUBROUTINE data_output_dvrp
